import chipwhisperer as cw

import numpy as np
import matplotlib.pyplot as plt

import time
import atexit

BITSTREAM_PATH = "./bitstream.bit"
TARGET_CLK = 1e6

TRACES = 10

scope = None
def disconnect_scope():
    try:
        scope.dis()
        del scope
        print("Sucessfully disconnected scope.")
    except Exception:
        pass


def main():
    global scope
    atexit.register(disconnect_scope)

    target = cw.target(None, cw.targets.CW305, bsfile=BITSTREAM_PATH, force=True)
    target.vccint_set(1.0)
        
    # we only need PLL1:
    target.pll.pll_enable_set(True)
    target.pll.pll_outenable_set(False, 0)
    target.pll.pll_outenable_set(True, 1)
    target.pll.pll_outenable_set(False, 2)

    # set target clock:
    target.pll.pll_outfreq_set(TARGET_CLK, 1)

    # 1ms is plenty of idling time
    target.clkusbautooff = True
    target.clksleeptime = 1

    print(f"Programmed FPGA with {BITSTREAM_PATH}.")

    scope = cw.scope()
    scope.adc.offset = 0
    scope.gain.db = 25
    scope.adc.samples = 20000
    scope.adc.basic_mode = "rising_edge"
    scope.trigger.triggers = "tio4"
    scope.io.hs2 = "disabled"
    scope.clock.adc_src = "extclk_dir"

    print("ADC parameters set. Start measuring...")

    traces_list = []

    for i in range(TRACES):
        scope.arm()
        time.sleep(0.1)
        assert not scope.capture()
        trace = scope.get_last_trace(as_int=True)
        traces_list.append(trace)

    traces = np.array(traces_list)

    print("Measuring donw. Start visualization...")
    
    plt.ylabel("ADC")
    plt.xlabel("Samples")
    plt.plot(traces.T)

    plt.savefig("traces.png")
    np.savez("traces.npz", traces=traces)

if __name__ == '__main__':
    main()
